## ---------------------------
##
## Run retrospective analysis
##
## ---------------------------


dir.create(file.path(base_dir, "retro"))

if(exists("iter_param")) {
  init_retro_param <- iter_param
} else if(exists("optim_param")) {
  init_retro_param <- optim_param
} else {
  init_retro_param <- tmb_param
}

retro_model <- list()
retro_params <- list()

for(peel in 0:5){
  
  retro_params[[peel+1]] <- init_retro_param
  retro_params[[peel+1]]$value$retro_years <- peel
  model_params$peel <- peel
  
  source("5 likelihood.R")
  
  if(force_bound_params) {
    retro_actions <- c(
      time_actions, stock_actions, fleet_actions, likelihood_actions,
      list(g3experiments::g3l_bounds_penalty(tmb_param))
    )
    
    retro_model[[peel+1]] <- g3_to_tmb(retro_actions)
  } else {
    retro_actions <- c(time_actions, stock_actions, fleet_actions, likelihood_actions)
    retro_model[[peel+1]] <- g3_to_tmb(retro_actions)
  }
}

retro_list <- parallel::mclapply(seq_along(retro_params), function(i) {
  
  print(i-1)
  # message("Running g3_optim for ", i-1)
  param <- g3_optim(model = retro_model[[i]],
                    params = retro_params[[i]],
                    use_parscale = TRUE,
                    method = 'BFGS',
                    control = list(maxit = maxit),
                    print_status = TRUE
  )
  #message("g3_optim for ", peel, " finished. Running g3_fit")
  
  fit <- g3_fit(retro_model[[i]], param)
  
  # message("Peel ", peel, " finished.")
  
  return(list(peel = i-1, param = param, fit = fit))
}, mc.cores = ifelse(ncores >= 5, 6, ncores)
)

### Save the model parameters

write.csv(lapply(retro_list, function(k) k$param) %>% bind_rows(), file = file.path(base_dir, "retro/Retro parameters.csv"))
save(retro_list, file = file.path(base_dir, "retro/Retro parameters and fit.rda"), compress = "xz")

# Plot

retro_fit <- lapply(retro_list, function(k) k$fit)

if(plot_retro_html) {
  message("Plotting retro html files. This will take a while.")
  tmppath <- file.path(getwd(), base_dir, "figures")
  lapply(seq_along(retro_fit), function(i) {
    make_html(
      retro_fit[[i]], path = tmppath,
      file_name = paste0("model_output_figures_retro_peel_", i-1, ".html")
    )
  })
  rm(tmppath)
}

ggsave(file.path(base_dir, "figures/Retro_model_biomass.png"),
       plot = gadgetplots::plot_retro(retro_fit),
       width = pagewidth, height = pagewidth*0.7, units = "mm")

