## ---------------------------
##
## Script name:
##
## Purpose of script:
##
## Author: Mikko Vihtakari // Institute of Marine Research, Norway
## Email: mikko.vihtakari@hi.no
##
## Date Created: 2024-01-05
##
## ---------------------------

## Source the run first script

source("0 run first.R")

## ---------------------------

## Load packages

## ---------------------------

## Source or list custom functions used within the script

## ---------------------------

## Read data

if(!exists("base_dir")) base_dir <- "latest_model"
outpath <- file.path(base_dir, "wknewref") # Output folder for the projections
if(!dir.exists(outpath)) dir.create(outpath, recursive = TRUE)

if(!exists("optim_fit")) load(file.path(base_dir, "optim", "Optimized TMB model fit.rda"))
fit <- optim_fit

## ---------------------------

# "stock.n": numbers at age at 1st of January per year and iteration
# "stock.wt": mean weight in the population along year per age, y ear and iteration

# plot_agecomp(fit, return_data = T) or

stock <- fit$stock.std %>%
  group_by(year, age) %>%
  reframe(n = sum(number), mean_weight = sum(number*mean_weight)/n)

ggplot(stock, aes(x = age, y = n)) +
  geom_col() +
  facet_wrap(~year)

ggplot(stock, aes(x = age, y = mean_weight)) +
  geom_col() +
  facet_wrap(~year)

# "m": total natural mortality rate per age, year and iteration

natural_mortality <- tibble(sex = c("female", "male"), M = c(0.12, 0.16))

# "catch.n": total catch by age, year and iteration
# "catch.wt": mean weight in the catch along year per age, year and iteration

catch <- fit$stock.prey %>%
  dplyr::select(year, stock, age, number_consumed) %>%
  left_join(
    fit$stock.std %>% dplyr::select(year, stock, age, mean_weight)
  ) %>%
  group_by(year, age) %>%
  reframe(catch_number = sum(number_consumed), mean_weight = sum(number_consumed*mean_weight)/catch_number)

ggplot(catch, aes(x = age, y = catch_number)) +
  geom_col() +
  facet_wrap(~year)

ggplot(catch, aes(x = age, y = mean_weight)) +
  geom_col() +
  facet_wrap(~year)

# "landings.n": total landings by age, year and iteration
# "landings.wt": mean weight in the landings along year per age, year and iteration
#  same than catch

# "discards.n": total discards by age, year and iteration
# "discards.wt": mean weight in the discards along year per age, year and iteration
# these are 0

# "mat": maturity  per age, year and iteration   at the time of spawning. ####

mat <- fit$stock.std %>%
  mutate(mature = ifelse(grepl("_mat$", stock), "mature", "immature"),
         sex = ifelse(grepl("_female_", stock), "female", "male")
  ) %>%
  group_by(year, age, sex, mature) %>%
  reframe(number = sum(number)) %>%
  pivot_wider(names_from = "mature", values_from = "number") %>%
  replace_na(list(immature = 0, mature = 0)) %>%
  mutate(ratio = mature/(mature + immature))

ggplot(mat, aes(x = age, y = ratio, color = sex)) +
  geom_path() +
  facet_wrap(~year)

# "harvest": total fishing mortality rate at age by year and iteration ####

harvest <- fit$stock.prey %>%
  # dplyr::filter(.data$age %in% fbar_ages) %>%
  dplyr::group_by(.data$year, .data$age) %>%
  dplyr::reframe(
    c = sum(.data$number_consumed),
    n = sum(.data$number[.data$step==1]),
    catch_mass = sum(.data$biomass_consumed)) %>%
  dplyr::mutate(f = -log(1 - .data$c/.data$n)) %>%
  dplyr::group_by(.data$year, .data$age) %>%
  dplyr::reframe(
    Fbar = mean(.data$f, na.rm = TRUE),
    catch_n = sum(.data$c, na.rm = TRUE),
    catch_mass = sum(.data$catch_mass, na.rm = TRUE))

ggplot(harvest, aes(x = age, y = Fbar)) +
  geom_path() +
  facet_wrap(~year)

# "harvest.spwn": proportion of fishing mortality occurring before spawning per age, year and iteration. = 1
# "harvest.spwn": proportion of natural mortality occurring before spawning per age, year and iteration = 1

# Save ####

save(stock, natural_mortality, catch, mat, harvest,
     file = file.path(outpath, 'nea-ghl-gadget-data-for-wknewref.Rdata'), compress = "xz")

## ####
