## ---------------------------
##
## Script name: Compile length, sex, and substock (maturity) distributions
##
## ---------------------------

## Source or list custom functions used within the script

source("R/figure_functions.R")
source("R/clean_data_functions.R")
source("R/split_g3_data.R")

## ---------------------------

# Read data

nor_survey_ldist <- readRDS("data/Length data for surveys.rds")
nor_survey_aldist <- readRDS("data/Age length data for surveys.rds")
nor_catch_ldist <- readRDS("data/Length data for catches.rds")
nor_catch_aldist <- readRDS("data/Age length data for catches.rds")

# Linf (move somewhere)
# nor_catch_ldist %>% # or nor_survey_ldist
#   filter(!is.na(sex), !is.na(length)) %>%
#   # filter(sampling_type %in% c("ENS", "ESS")) %>%
#   # filter(gear %in% "BottomTrawls") %>%
#   group_by(sex) %>%
#   reframe(quantile = c(0.01, 0.05, 0.25, 0.5, 0.75, 0.95, 0.99),
#             length = quantile(length, quantile))

##############
# Catches ####

## TrawlNor ####

message("TrawlNor")

### ldist ####

TrawlNor_ldist <-
  g3_data(
    nor_catch_ldist %>%
      filter(gear %in% c("BottomTrawls", "PelagicTrawls", "Seines", "DanishSeines")),
    params =
      list(year = model_params$year_range,
           timestep = model_params$timestep_fun,
           length = mfdb_interval(
             "len",
             seq(stock_params$minlength, stock_params$maxlength,
                 by = 2*stock_params$dl),
             open_ended = c("upper","lower")
           )
      )
  ) %>%
  filter(!year %in% c(1991, 1993, 1995, 1996, 1998))

test <- TrawlNor_ldist %>%
  group_by(year, step) %>%
  summarise(n = sum(number)) %>%
  filter(n < 20)

if(nrow(test) > 0) {
  warning("Following year-step in TrawlNor_ldist contain < 10 fish: ", paste(paste(test$year, test$step, sep = "-"), collapse = ", "))
} else {
  rm(test)
}

png(file.path(base_dir, "figures/TrawlNor_ldist.png"), width = pagewidth, height = pagewidth*1.5, units = "mm", res = 300)
print(plot.ldist(TrawlNor_ldist, type = "ggridges"))
dev.off()

### Sex ratio ####

TrawlNor_sexratio <- g3_data(
  nor_catch_ldist %>%
    filter(gear %in% c("BottomTrawls", "PelagicTrawls", "Seines", "DanishSeines"),
           !is.na(sex),
           length >= 46),
  params =
    list(year = model_params$year_range,
         timestep = model_params$timestep_fun,
         sex = mfdb_group(female = 'F', male = 'M'),
         length = mfdb_interval(
           'len',
           seq(46, stock_params$male_mat$max_possible_data_length+6,
               by = 5*stock_params$dl),
           open_ended = c('upper')
         )
    )
) %>%
  filter(!year %in% c(1981, 1988, 1992, 1994, 1995, 1997, 1998, 1999, 2000, 2001, 2002, 2005, 2011))

TrawlNor_sexratio <- clean_sexratio_data(TrawlNor_sexratio)

png(file.path(base_dir, "figures/TrawlNor_sexratio.png"), width = pagewidth, height = pagewidth*1.5, units = "mm", res = 300)
print(plot.sexr(TrawlNor_sexratio))
dev.off()

### Split length distributions ####

TrawlNor_split_sexratio <- g3_data(
  nor_catch_ldist %>%
    filter(gear %in% c("BottomTrawls", "PelagicTrawls", "Seines", "DanishSeines"),
           !is.na(sex)),
  params =
    list(year = model_params$year_range,
         timestep = model_params$timestep_fun,
         sex = mfdb_group(female = 'F', male = 'M'),
         length =
           mfdb_interval(
             "len",
             seq(stock_params$minlength, stock_params$maxlength,
                 by = 2*stock_params$dl),
             open_ended = c("upper","lower")
           )
    )
) %>%
  filter(!year %in% c(1981, 1988, 1992, 1994, 1995, 1997, 1998, 1999, 2000, 2001, 2002, 2005, 2011))

TrawlNor_split_sexratio <-
  clean_sexratio_data(TrawlNor_split_sexratio)

png(file.path(base_dir, "figures/TrawlNor_ldist_sex_splitting.png"), width = pagewidth*1.2, height = pagewidth*1.2, units = "mm", res = 300)
TrawlNor_ldist_sex <- split_g3_data(TrawlNor_ldist, TrawlNor_split_sexratio, split_column = "sex", method = "mean", separate = FALSE, plot = TRUE)
dev.off()


## OtherNor ####

message("OtherNor")

### ldist ####

OtherNor_ldist <- g3_data(
  nor_catch_ldist %>%
    filter(gear %in% c("Longlines", "Gillnets")),
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length = mfdb_interval(
        "len",
        seq(stock_params$minlength, stock_params$maxlength,
            by = 2*stock_params$dl),
        open_ended = c("upper","lower")
      )
    )
)

test <- OtherNor_ldist %>%
  group_by(year, step) %>%
  summarise(n = sum(number)) %>%
  filter(n < 50)

if(nrow(test) > 0) {
  warning("Following year-step in OtherNor_ldist contain < 10 fish: ", paste(paste(test$year, test$step, sep = "-"), collapse = ", "))
} else {
  rm(test)
}

png(file.path(base_dir, "figures/OtherNor_ldist.png"), width = pagewidth, height = pagewidth*1.5, units = "mm", res = 300)
print(plot.ldist(OtherNor_ldist, type = "ggridges"))
dev.off()

### Sex ratio ####

OtherNor_sexratio <- g3_data(
  nor_catch_ldist %>%
    filter(gear %in% c("Longlines", "Gillnets"),
           !is.na(sex),
           length >= 46),
  params =
    list(year = model_params$year_range,
         timestep = model_params$timestep_fun,
         sex = mfdb_group(female = 'F', male = 'M'),
         length = mfdb_interval(
           'len',
           seq(46, stock_params$male_mat$max_possible_data_length+6,
               by = 5*stock_params$dl),
           open_ended = c('upper')
         )
    )
) %>% filter(!year %in% c(1981,1985,1991,1994,2011,2012))

OtherNor_sexratio <- clean_sexratio_data(OtherNor_sexratio)

png(file.path(base_dir, "figures/OtherNor_sexratio.png"), width = pagewidth, height = pagewidth*1.5, units = "mm", res = 300)
print(plot.sexr(OtherNor_sexratio))
dev.off()

### Split length distributions ####

OtherNor_split_sexratio <- g3_data(
  nor_catch_ldist %>%
    filter(gear %in% c("Longlines", "Gillnets"),
           !is.na(sex)),
  params =
    list(year = model_params$year_range,
         timestep = model_params$timestep_fun,
         sex = mfdb_group(female = 'F', male = 'M'),
         length =
           mfdb_interval(
             "len",
             seq(stock_params$minlength, stock_params$maxlength,
                 by = 2*stock_params$dl),
             open_ended = c("upper","lower")
           )
    )
) %>%
  filter(!year %in% c(1981,1985,1991,1994,2011,2012))

OtherNor_split_sexratio <- clean_sexratio_data(OtherNor_split_sexratio)

png(file.path(base_dir, "figures/OtherNor_ldist_sex_splitting.png"), width = pagewidth*1.2, height = pagewidth*1.2, units = "mm", res = 300)
OtherNor_ldist_sex <- split_g3_data(OtherNor_ldist, OtherNor_split_sexratio, split_column = "sex", separate = FALSE, plot = TRUE)
dev.off()

### aldist ####

OtherNor_aldist <- g3_data(
  nor_catch_aldist %>% filter(
    gear %in% c("Longlines", "Gillnets"),
    !is.na(age),
    !is.na(sex),
    sex %in% c("F", "M")),
  params =
    list(
      timestep = model_params$timestep_fun,
      age = mfdb_interval(
        "age", stock_params$minage:stock_params$maxage,
        open_ended = c("upper")
      ),
      length = mfdb_interval(
        "len",
        seq(41, stock_params$male_mat$max_possible_data_length+6, by = 5),
        open_ended = c("upper")
      ),
      sex = mfdb_group(female = 'F', male = 'M')
    ),
  method = "est_n",
  verbose = FALSE
) %>% rename("number" = "est_n") %>% filter(!year %in% c(2019))


png(file.path(base_dir, "figures/OtherNor_aldist.png"), width = pagewidth, height = pagewidth*2, units = "mm", res = 300)
p1 <- OtherNor_aldist %>% filter(sex == "female") %>% plot.aldist(., facet_age = FALSE, ncol = 1, scales = "free_y") + ggtitle("Females") + theme(legend.position = "bottom")
p2 <- OtherNor_aldist %>% filter(sex == "male") %>% plot.aldist(., facet_age = FALSE, ncol = 1, scales = "free_y") + ggtitle("Males") + theme(legend.position = "none")
cowplot::plot_grid(cowplot::plot_grid(p1 + theme(legend.position = "none"), p2), cowplot::get_legend(p1), ncol = 1, rel_heights = c(10,1)) %>% print
dev.off()

rm(p1, p2)

## TrawlRus ####

message("TrawlRus")

### ldist ####
## Before 2017

tmp_f <- readRDS("data/Russian trawl ldist female from gadget2.rds")
tmp_m <- readRDS("data/Russian trawl ldist male from gadget2.rds")

if(identical(model_params$timestep_fun, mfdb_timestep_yearly)) {
  tmp_f$step <- 1
  tmp_m$step <- 1
}

split_fun <- function(x) {
  mean(
    c(as.numeric(sapply(strsplit(x, "-"), "[", 2)),
      as.numeric(sapply(strsplit(x, "-"), "[", 3)))
  )
}

tmp_f <- tmp_f %>%
  mutate(
    sex = "female",
    length = sapply(length, split_fun))

tmp_m <- tmp_m %>%
  mutate(
    sex = "male",
    length = sapply(length, split_fun))

## After 2017

tmp <- lapply(readxl::excel_sheets("data/RUS GHL lenght catches 2017- 2022.xlsx"), function(k) {
  tmp <- suppressMessages(readxl::read_excel("data/RUS GHL lenght catches 2017- 2022.xlsx", k))

  tmp <- tmp %>%
    mutate(
      area = tmp %>% pull(3) %>% ifelse(grepl("AREA",.), ., NA) %>% zoo::na.locf(),
      .after = 3
    )

  tmp <- tmp[!grepl("AREA",tmp[[3]]),]
  tmp <- tmp[!is.na(tmp[[3]]),]

  colnames(tmp)[1:3] <- c("year", "step", "sex")

  tmp %>%
    mutate(year = zoo::na.locf(year),
           step = zoo::na.locf(step)) %>%
    filter(!grepl("TOTAL", area)) %>%
    filter(!grepl("year", step)) %>%
    mutate(step = gsub(" quarter", "", step)) %>%
    dplyr::select(-SUM) %>%
    tidyr::pivot_longer(cols = c(-year, -step, -sex, -area), names_to = "length") %>%
    group_by(year, sex, length) %>%
    summarise(number = sum(value, na.rm = TRUE)) %>%
    mutate(length = as.integer(length)) %>%
    arrange(year, sex, length) %>%
    mutate(step = 1, area = "all", .after = year) %>%
    mutate(sex = ifelse(sex == "F", "female", ifelse(sex == "M", "male", "ERROR")))
}) %>% bind_rows()

## Combine

tmp <- bind_rows(tmp_f, tmp_m) %>%
  dplyr::select(year, step, area, sex, length, number) %>%
  bind_rows(tmp)

TrawlRus_ldist <- gadgetutils::g3_data(
  tmp,
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length = mfdb_interval(
        "len",
        seq(31, 95, by = 2*stock_params$dl),
        open_ended = c("upper","lower")
      )
    ),
  method = "number",
  column_names = c(year = "year", step = "step", area = "area"),
  verbose = FALSE) %>%
  filter(!year %in% c(1991, 1993, 2019))

png(file.path(base_dir, "figures/TrawlRus_ldist.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.ldist(TrawlRus_ldist, type = "ggridges"))
dev.off()

### Sex-split ldists ####

TrawlRus_ldist_sex <-
  gadgetutils::g3_data(
    tmp,
    params =
      list(
        year = model_params$year_range,
        timestep = model_params$timestep_fun,
        length = mfdb_interval(
          "len",
          seq(31, 95, by = 2*stock_params$dl),
          open_ended = c("upper","lower")
        ),
        sex = mfdb_group(female = 'female', male = 'male')
      ),
    method = "number",
    column_names = c(year = "year", step = "step", area = "area"),
    verbose = FALSE) %>%
  filter(!year %in% c(1991, 1993))

TrawlRus_ldist_female <- gadgetutils::g3_data(
  tmp_f,
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length = mfdb_interval(
        "len",
        seq(stock_params$minlength, stock_params$maxlength,
            by = 2*stock_params$dl),
        open_ended = c("upper","lower")
      )
    ),
  method = "number",
  column_names = c(year = "year", step = "step", area = "area"),
  verbose = FALSE) %>%
  filter(!year %in% c(1991, 1993))

png(file.path(base_dir, "figures/TrawlRus_ldist_female.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.ldist(TrawlRus_ldist_female))
dev.off()

TrawlRus_ldist_male <- gadgetutils::g3_data(
  tmp_m,
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length = mfdb_interval(
        "len",
        seq(stock_params$minlength, stock_params$maxlength,
            by = 2*stock_params$dl),
        open_ended = c("upper","lower")
      )
    ),
  method = "number",
  column_names = c(year = "year", step = "step", area = "area"),
  verbose = FALSE) %>%
  filter(!year %in% c(1991, 1993))

png(file.path(base_dir, "figures/TrawlRus_ldist_male.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.ldist(TrawlRus_ldist_male))
dev.off()

### Sex ratio ####

# tmp <- tmp %>% filter(length >= tmp %>% filter(number>0) %>% pull(length) %>% min())

TrawlRus_sexratio <- g3_data(
  tmp %>% filter(length >= 36),
  params =
    list(
      length =
        mfdb_interval(
          'len',
          seq(36, stock_params$male_mat$max_possible_data_length+6,
              by = 5*stock_params$dl),
          open_ended = c('upper')),
      sex = mfdb_group(female = 'female', male = 'male'),
      timestep = model_params$timestep_fun,
      year = model_params$year_range
    ),
  method = "number",
  column_names = c(year = "year", step = "step", area = "area")
)

TrawlRus_sexratio <- clean_sexratio_data(TrawlRus_sexratio)

png(file.path(base_dir, "figures/TrawlRus_sexratio.png"), width = pagewidth, height = pagewidth*1.5, units = "mm", res = 300)
print(plot.sexr(TrawlRus_sexratio))
dev.off()

## OtherRus ####

message("OtherRus")

### ldist ####

tmp_f <- readRDS("data/Russian other ldist female from gadget2.rds")
tmp_m <- readRDS("data/Russian other ldist male from gadget2.rds")

if(identical(model_params$timestep_fun, mfdb_timestep_yearly)) {
  tmp_f$step <- 1
  tmp_m$step <- 1
}

split_fun <- function(x) {
  mean(
    c(as.numeric(sapply(strsplit(x, "-"), "[", 2)),
      as.numeric(sapply(strsplit(x, "-"), "[", 3)))
  )
}

tmp_f <- tmp_f %>%
  mutate(
    sex = "female",
    length = sapply(length, split_fun))

tmp_m <- tmp_m %>%
  mutate(
    sex = "male",
    length = sapply(length, split_fun))

tmp <- bind_rows(tmp_f, tmp_m) %>%
  dplyr::select(year, step, area, sex, length, number)

OtherRus_ldist <- gadgetutils::g3_data(
  tmp,
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length =
        mfdb_interval(
          "len",
          seq(stock_params$minlength, stock_params$maxlength,
              by = 2*stock_params$dl),
          open_ended = c("upper","lower")
        )
    ),
  method = "number",
  column_names = c(year = "year", step = "step", area = "area"),
  verbose = FALSE) %>%
  filter(!year %in% c(1995, 2005))

png(file.path(base_dir, "figures/OtherRus_ldist.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.ldist(OtherRus_ldist, type = "ggridges"))
dev.off()

### Sex-split ldists ####

OtherRus_ldist_sex <-
  gadgetutils::g3_data(
    tmp,
    params =
      list(
        year = model_params$year_range,
        timestep = model_params$timestep_fun,
        length = mfdb_interval(
          "len",
          seq(stock_params$minlength, stock_params$maxlength,
              by = 2*stock_params$dl),
          open_ended = c("upper","lower")
        ),
        sex = mfdb_group(female = 'female', male = 'male')
      ),
    method = "number",
    column_names = c(year = "year", step = "step", area = "area"),
    verbose = FALSE) %>%
  filter(!year %in% c(1995, 2005))

OtherRus_ldist_female <- gadgetutils::g3_data(
  tmp_f,
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length = mfdb_interval(
        "len",
        seq(stock_params$minlength, stock_params$maxlength,
            by = 2*stock_params$dl),
        open_ended = c("upper","lower")
      )
    ),
  method = "number",
  column_names = c(year = "year", step = "step", area = "area"),
  verbose = FALSE) %>%
  filter(!year %in% c(1995, 2005))

png(file.path(base_dir, "figures/OtherRus_ldist_female.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.ldist(OtherRus_ldist_female))
dev.off()

OtherRus_ldist_male <- gadgetutils::g3_data(
  tmp_m,
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length = mfdb_interval(
        "len",
        seq(stock_params$minlength, stock_params$maxlength,
            by = 2*stock_params$dl),
        open_ended = c("upper","lower")
      )
    ),
  method = "number",
  column_names = c(year = "year", step = "step", area = "area"),
  verbose = FALSE) %>%
  filter(!year %in% c(1995, 2005))

png(file.path(base_dir, "figures/OtherRus_ldist_male.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.ldist(OtherRus_ldist_male))
dev.off()

### Sex ratio ####

OtherRus_sexratio <- g3_data(
  tmp %>% filter(length >= 36),
  params =
    list(
      length =
        mfdb_interval(
          'len',
          seq(36, stock_params$male_mat$max_possible_data_length+6,
              by = 5*stock_params$dl),
          open_ended = c('upper')),
      sex = mfdb_group(female = 'female', male = 'male'),
      timestep = model_params$timestep_fun,
      year = model_params$year_range
    ),
  method = "number",
  column_names = c(year = "year", step = "step", area = "area"),
  verbose = FALSE
) %>%
  filter(!year %in% c(1995, 2005))

OtherRus_sexratio <- clean_sexratio_data(OtherRus_sexratio)

png(file.path(base_dir, "figures/OtherRus_sexratio.png"), width = pagewidth, height = pagewidth*1.5, units = "mm", res = 300)
print(plot.sexr(OtherRus_sexratio))
dev.off()

##############
# Surveys ####

#############
## EggaN ####

message("EggaN")

### ldist ####

EggaN_ldist <- g3_data(
  nor_survey_ldist %>%
    filter(
      sampling_type == "ENS",
      gear == "BottomTrawls"
    ),
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length = mfdb_interval(
        "len",
        seq(stock_params$minlength, stock_params$maxlength,
            by = 2*stock_params$dl),
        open_ended = c("upper","lower")
      )
    ),
  verbose = FALSE
)

png(file.path(base_dir, "figures/EggaN_ldist.png"), width = pagewidth, height = pagewidth*1.5, units = "mm", res = 300)
print(plot.ldist(EggaN_ldist, type = "ggridges"))
dev.off()

### aldist ####

EggaN_aldist_female <- g3_data(
  nor_survey_aldist %>%
    filter(
      sampling_type == "ENS",
      sex == "F",
      !is.na(age)
    ),
  params =
    list(
      timestep = model_params$timestep_fun,
      age = mfdb_interval(
        "age", stock_params$minage:stock_params$maxage,
        open_ended = c("upper")
      ),
      length = mfdb_interval(
        "len",
        seq(26, 87, by = 5),
        open_ended = c("upper")
      )
    ),
  method = "est_n",
  verbose = FALSE
) %>% rename("number" = "est_n") %>% filter(!year %in% c(2001,2007))

png(file.path(base_dir, "figures/EggaN_aldist_female.png"), width = pagewidth, height = pagewidth*2, units = "mm", res = 300)
print(plot.aldist(EggaN_aldist_female, scales = "free_y"))
dev.off()

png(file.path(base_dir, "figures/EggaN_adist_female.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.adist(EggaN_aldist_female, scales = "free_y", ncol = 1))
dev.off()

EggaN_aldist_male <- g3_data(
  nor_survey_aldist %>%
    filter(
      sampling_type == "ENS",
      sex == "M",
      !is.na(age)
    ),
  params =
    list(
      timestep = model_params$timestep_fun,
      age = mfdb_interval(
        "age", stock_params$minage:stock_params$maxage,
        open_ended = c("upper")
      ),
      length = mfdb_interval(
        "len",
        seq(26,66, by = 5),
        open_ended = c("upper")
      )
    ),
  method = "est_n",
  verbose = FALSE
) %>% rename("number" = "est_n") %>% filter(!year %in% c(2001,2007))


png(file.path(base_dir, "figures/EggaN_aldist_male.png"), width = pagewidth, height = pagewidth*2, units = "mm", res = 300)
print(plot.aldist(EggaN_aldist_male))
dev.off()

png(file.path(base_dir, "figures/EggaN_adist_male.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.adist(EggaN_aldist_male, scales = "free_y", ncol = 1))
dev.off()

### matp ####

EggaN_mat <- mfdb_concatenate_results(
  g3_data(
    x = nor_survey_ldist %>%
      filter(
        sampling_type == "ENS",
        gear == "BottomTrawls",
        !is.na(sex),
        !is.na(maturity_stage),
        sex == "M"
      ),
    params = list(
      length =
        mfdb_interval(
          'len', seq(31, 91, by = 5*stock_params$dl),
          open_ended = c('lower','upper')),
      maturity_stage = mfdb_group(male_imm = 1:2, male_mat = 3:5),
      timestep = model_params$timestep_fun,
      year = model_params$year_range
    )
  ),
  g3_data(
    x = nor_survey_ldist %>%
      filter(
        sampling_type == "ENS",
        gear == "BottomTrawls",
        !is.na(sex),
        !is.na(maturity_stage),
        sex == "F"
      ),
    params = list(
      length =
        mfdb_interval(
          'len', seq(31, 91, by = 5*stock_params$dl),
          open_ended = c('lower','upper')),
      maturity_stage = mfdb_group(female_imm = 1:2, female_mat = 3:5),
      timestep = model_params$timestep_fun,
      year = model_params$year_range
    )
  )
) %>% filter(!year %in% 1999)

EggaN_mat <- clean_mat_data(EggaN_mat)

png(file.path(base_dir, "figures/EggaN_maturity_data.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.mat(EggaN_mat))
dev.off()

png(file.path(base_dir, "figures/EggaN_maturity_data_proportions.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.matp(EggaN_mat))
dev.off()

png(file.path(base_dir, "figures/Maturity_age_ldist_length_comparison.png"), width = pagewidth, height = pagewidth*0.7, units = "mm", res = 300)
print(compare_mat_ldist(EggaN_ldist, EggaN_mat, rbind(EggaN_aldist_female %>% mutate(sex = "F"), EggaN_aldist_male %>% mutate(sex = "M"))))
dev.off()


#############
## EggaS ####

### Ldist ####

EggaS_ldist <- g3_data(
  nor_survey_ldist %>%
    filter(
      sampling_type == "ESS",
      gear == "BottomTrawls"
    ),
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length = mfdb_interval(
        "len",
        seq(stock_params$minlength, stock_params$maxlength,
            by = 2*stock_params$dl),
        open_ended = c("upper","lower")
      )
    )
)

png(file.path(base_dir, "figures/EggaS_ldist.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.ldist(EggaS_ldist, type = "ggridges"))
dev.off()

### Aldist ####

EggaS_aldist <- g3_data(
  nor_survey_aldist %>%
    filter(
      sampling_type == "ESS",
      !is.na(age)
    ),
  params =
    list(
      timestep = model_params$timestep_fun,
      age = mfdb_interval(
        "age", stock_params$minage:stock_params$maxage,
        open_ended = c("upper")
      ),
      length = mfdb_interval(
        "len",
        seq(26, 87, by = 5),
        open_ended = c("upper")
      ),
      sex = mfdb_group(female = 'F', male = 'M')
    ),
  method = "est_n",
  verbose = FALSE
) %>% rename("number" = "est_n")

png(file.path(base_dir, "figures/EggaS_aldist.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
p1 <- EggaS_aldist %>% filter(sex == "female") %>% plot.aldist(., facet_age = FALSE, ncol = 1, scales = "free_y") + ggtitle("Females") + theme(legend.position = "bottom")
p2 <- EggaS_aldist %>% filter(sex == "male") %>% plot.aldist(., facet_age = FALSE, ncol = 1, scales = "free_y") + ggtitle("Males") + theme(legend.position = "none")
cowplot::plot_grid(cowplot::plot_grid(p1 + theme(legend.position = "none"), p2), cowplot::get_legend(p1), ncol = 1, rel_heights = c(10,1)) %>% print
dev.off()

rm(p1, p2)

### Sex ratio ####

# EggaS_sexratio <- g3_data(
#   x = nor_survey_ldist %>%
#     filter(
#       sampling_type == "ESS",
#       gear == "BottomTrawls",
#       !is.na(sex)
#     ),
#   params = list(
#     length =
#       mfdb_interval(
#         'len',
#         seq(41, stock_params$male_mat$max_possible_data_length+6,
#             by = 5*stock_params$dl),
#         open_ended = c('upper')),
#     sex = mfdb_group(female = 'F', male = 'M'),
#     timestep = model_params$timestep_fun,
#     year = model_params$year_range
#   )
# )
#
# EggaS_sexratio <- clean_sexratio_data(EggaS_sexratio)
#
# png(file.path(base_dir, "figures/EggaS_sexratio.png"), width = pagewidth, height = pagewidth*1.5, units = "mm", res = 300)
# print(plot.sexr(EggaS_sexratio))
# dev.off()

### Matp ####

EggaS_mat <- mfdb_concatenate_results(
  g3_data(
    x = nor_survey_ldist %>%
      filter(
        sampling_type == "ESS",
        gear == "BottomTrawls",
        !is.na(sex),
        !is.na(maturity_stage),
        sex == "M"
      ),
    params = list(
      length =
        mfdb_interval(
          'len', seq(31, 91, by = 5*stock_params$dl),
          open_ended = c('lower','upper')),
      maturity_stage = mfdb_group(male_imm = 1:2, male_mat = 3:5),
      timestep = model_params$timestep_fun,
      year = model_params$year_range
    )
  ),
  g3_data(
    x = nor_survey_ldist %>%
      filter(
        sampling_type == "ESS",
        gear == "BottomTrawls",
        !is.na(sex),
        !is.na(maturity_stage),
        sex == "F"
      ),
    params = list(
      length =
        mfdb_interval(
          'len', seq(31, 91, by = 5*stock_params$dl),
          open_ended = c('lower','upper')),
      maturity_stage = mfdb_group(female_imm = 1:2, female_mat = 3:5),
      timestep = model_params$timestep_fun,
      year = model_params$year_range
    )
  )
)

EggaS_mat <- clean_mat_data(EggaS_mat) %>% filter(!year %in% 2016)

png(file.path(base_dir, "figures/EggaS_maturity_data.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.mat(EggaS_mat))
dev.off()

png(file.path(base_dir, "figures/EggaS_maturity_data_proportions.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.matp(EggaS_mat))
dev.off()

########################
## Ecosystem survey ####

message("EcoS")

### ldist ####

EcoS_ldist <- g3_data(
  nor_survey_ldist %>%
    filter(
      sampling_type == "ECS",
      gear == "ShrimpTrawls"
    ),
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length = mfdb_interval(
        "len",
        seq(stock_params$minlength, stock_params$maxlength,
            by = 2*stock_params$dl),
        open_ended = c("upper","lower")
      )
    )
) %>% filter(!year %in% c(2014))


png(file.path(base_dir, "figures/EcoS_ldist.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.ldist(EcoS_ldist, type = "ggridges"))
dev.off()

### Age-length ####

EcoS_aldist <- g3_data(
  nor_survey_aldist %>%
    filter(
      sampling_type == "ECS",
      !is.na(age),
      !is.na(sex)
    ) %>%
    mutate(sex = ifelse(
      sex == "U" & length_bin %in% c("(0,5]", "(5,10]", "(10,15]", "(15,20]", "(20,25]"),
      sample(c("F", "M"), replace = TRUE), sex),
      age = ifelse(age == 0, 1, age)) %>%
    filter(sex %in% c("F", "M")),
  params =
    list(
      timestep = model_params$timestep_fun,
      age = mfdb_interval(
        "age", 0:stock_params$maxage,
        open_ended = c("upper")
      ),
      length = mfdb_interval(
        "len", seq(11, 76, by = 5),
        open_ended = c("upper","lower")
      ),
      sex = mfdb_group(female = 'F', male = 'M')
    ),
  method = "est_n",
  verbose = FALSE
) %>%
  rename("number" = "est_n") %>%
  filter(!year %in% c(2013, 2014, 2015, 2016, 2017, 2019))

png(file.path(base_dir, "figures/EcoS_aldist.png"), width = pagewidth, height = pagewidth*1.5, units = "mm", res = 300)
p1 <- EcoS_aldist %>% filter(sex == "female") %>% plot.aldist(., facet_age = FALSE, ncol = 1, scales = "free_y") + ggtitle("Females") + theme(legend.position = "bottom")
p2 <- EcoS_aldist %>% filter(sex == "male") %>% plot.aldist(., facet_age = FALSE, ncol = 1, scales = "free_y") + ggtitle("Males") + theme(legend.position = "none")
cowplot::plot_grid(cowplot::plot_grid(p1 + theme(legend.position = "none"), p2), cowplot::get_legend(p1), ncol = 1, rel_heights = c(10,1)) %>% print
dev.off()

rm(p1, p2)

### Sex ratio ####

EcoS_sexratio <- g3_data(
  x = nor_survey_ldist %>%
    filter(
      sampling_type == "ECS",
      gear == "ShrimpTrawls",
      !is.na(sex)
    ),
  params = list(
    length =
      mfdb_interval(
        'len',
        seq(21, stock_params$male_mat$max_possible_data_length+6,
            by = 5*stock_params$dl),
        open_ended = c('lower','upper')),
    sex = mfdb_group(female = 'F', male = 'M'),
    timestep = model_params$timestep_fun,
    year = model_params$year_range
  )
)

EcoS_sexratio <- clean_sexratio_data(EcoS_sexratio)

png(file.path(base_dir, "figures/EcoS_sexratio.png"), width = pagewidth, height = pagewidth*1.5, units = "mm", res = 300)
print(plot.sexr(EcoS_sexratio))
dev.off()

#####################
## Winter survey ####

message("WinterS")

WinterS_ldist <- g3_data(
  nor_survey_ldist %>%
    filter(
      sampling_type == "WNS",
      gear == "ShrimpTrawls",
      population %in% c("H100-300","H300-400","I100-300","I300-400","I400-500",
                        "J100-300","J300-400","L100-300","L300-400"),
      year %in% model_params$year_range[!model_params$year_range %in% "1980"]
    ),
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length = mfdb_interval(
        "len",
        seq(stock_params$minlength, stock_params$maxlength,
            by = 5*stock_params$dl),
        open_ended = c("upper","lower")
      )
    )
)

png(file.path(base_dir, "figures/WinterS_ldist.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.ldist(WinterS_ldist, type = "ggridges"))
dev.off()

######################
## Russian survey ####

message("RussianS")

### Ldist ####

tmp <-
  readxl::read_excel("data/Russian survey index data.xlsx", "F slope") %>%
  filter(!is.na(`20.0-20.9`)) %>%
  dplyr::select(-abundance, -biomass) %>%
  pivot_longer(cols = -year, names_to = "length", values_to = "number") %>%
  mutate(
    length = as.numeric(sapply(strsplit(length, "-"), "[", 1))
  ) %>%
  bind_rows(
    readxl::read_excel("data/Russian survey index data.xlsx", "M slope") %>%
      filter(!is.na(`20.0-20.9`)) %>%
      dplyr::select(-abundance, -biomass) %>%
      pivot_longer(cols = -year, names_to = "length", values_to = "number") %>%
      mutate(
        length = as.numeric(sapply(strsplit(length, "-"), "[", 1))
      )
  ) %>%
  group_by(year, length) %>%
  summarise(number = sum(number)) %>%
  arrange(year, length) %>%
  ungroup() %>%
  mutate(step = 1, area= "all", .before = "length")

RussianS_ldist <- g3_data(
  x = tmp,
  params =
    list(
      year = model_params$year_range,
      timestep = model_params$timestep_fun,
      length = mfdb_interval(
        "len",
        seq(31, max(tmp$length),
            by = 2*stock_params$dl),
        open_ended = c("upper")
      )
    ),
  column_names = c("year" = "year", "step" = "step", "area" = "area"),
  method = "number",
  verbose = FALSE
)

png(file.path(base_dir, "figures/RussianS_ldist.png"), width = pagewidth, height = pagewidth, units = "mm", res = 300)
print(plot.ldist(RussianS_ldist, type = "ggridges"))
dev.off()

### Sex ratio ####

tmp <-
  readxl::read_excel("data/Russian survey index data.xlsx", "F slope") %>%
  filter(!is.na(`20.0-20.9`)) %>%
  dplyr::select(-abundance, -biomass) %>%
  pivot_longer(cols = -year, names_to = "length", values_to = "number") %>%
  mutate(length = as.numeric(sapply(strsplit(length, "-"), "[", 1))) %>%
  add_column(sex = "F", .before = "length") %>%
  bind_rows(
    readxl::read_excel("data/Russian survey index data.xlsx", "M slope") %>%
      filter(!is.na(`20.0-20.9`)) %>%
      dplyr::select(-abundance, -biomass) %>%
      pivot_longer(cols = -year, names_to = "length", values_to = "number") %>%
      mutate(length = as.numeric(sapply(strsplit(length, "-"), "[", 1))) %>%
      add_column(sex = "M", .before = "length")
  ) %>%
  arrange(year, sex, length) %>%
  mutate(step = 1, area= "all", .before = "length")

RussianS_sexratio <- g3_data(
  x = tmp,
  params = list(
    length = mfdb_interval(
      'len', seq(31, 91, by = 5*stock_params$dl),
      open_ended = c('upper')),
    sex = mfdb_group(female = 'F', male = 'M'),
    timestep = model_params$timestep_fun,
    year = model_params$year_range
  ),
  column_names = c("year" = "year", "step" = "step", "area" = "area"),
  method = "number"
)

RussianS_sexratio <- clean_sexratio_data(RussianS_sexratio)

png(file.path(base_dir, "figures/RussianS_sexratio.png"), width = pagewidth, height = pagewidth*1.5, units = "mm", res = 300)
print(plot.sexr(RussianS_sexratio))
dev.off()


###########
# Save ####

save(TrawlNor_ldist, TrawlNor_sexratio, TrawlNor_ldist_sex, OtherNor_ldist, OtherNor_sexratio, OtherNor_ldist_sex, OtherNor_aldist, TrawlRus_ldist, TrawlRus_sexratio, TrawlRus_ldist_sex, OtherRus_ldist, OtherRus_sexratio, OtherRus_ldist_sex, EggaN_ldist, EggaN_aldist_female, EggaN_aldist_male, EggaN_mat, EggaS_ldist, EggaS_aldist, EggaS_mat, EcoS_ldist, EcoS_aldist, EcoS_sexratio, WinterS_ldist, RussianS_ldist, RussianS_sexratio,
     file = file.path(base_dir, "data/Catch distributions to Gadget.rda"))

