## ---------------------------
##
## Optimize a gadget model parameters
##
## ---------------------------

dir.create(file.path(base_dir, "optim"))

if(exists("iter_param")) {
  init_optim_param <- iter_param
} else {
  init_optim_param <- tmb_param
}

if(nrow(init_optim_param %>% filter(optimise, lower >= upper)) > 0) warning("Parameter lower bounds higher than upper bounds. Expect trouble in optimization.")

time_optim_start <- Sys.time()
message("Optimization started ", time_optim_start)
## g3_optim is a wrapper for stats::optim. It returns the parameter
## dataframe with the optimised parameters and includes an attribute with
## a summary of the optimisation.
## The control argument is identical to control for optim
optim_param <- g3_optim(model = tmb_model,
                        params = init_optim_param,
                        use_parscale = TRUE,
                        method = 'BFGS',
                        control = list(maxit = maxit #, reltol = 1e-9
                        ),
                        print_status = TRUE
)
time_optim_end <- Sys.time()
time_optim <- round(as.numeric(time_optim_end - time_optim_start, units = "mins"), 1)
message("Optimization finished ", time_optim_end, " after ", time_optim, " min")

## Write the times to a file
if(!file.exists(file.path(base_dir, "session/run_times.txt"))) {
  info_file <- file(file.path(base_dir, "session/run_times.txt"))
  close(info_file)
}

cat(
  c("Optimization:\n",
    "   started ", as.character(time_optim_start), "\n",
    "   finished ", as.character(time_optim_end), "\n",
    "   time ", time_optim, " min", "\n",
    "   method: ", attributes(optim_param)$summary$method, "\n",
    "   convergence: ", attributes(optim_param)$summary$convergence, "\n",
    "   iterations: ", attributes(optim_param)$summary$gd_calls, "\n",
    "   score: ", round(attributes(optim_param)$summary$score, 1), "\n\n"),
  file = file.path(base_dir, "session/run_times.txt"), sep = "")

### Save the model parameters

write.csv(as.data.frame(optim_param), file = file.path(base_dir, "optim/Optimized TMB parameters.csv"))
save(optim_param, file = file.path(base_dir, "optim/Optimized TMB parameters.rda"), compress = "xz")

## Plots

optim_fit <- g3_fit(model, optim_param)
save(optim_fit, file = file.path(base_dir, "optim/Optimized TMB model fit.rda"), compress = "xz")

if(plot_html) {
  tmppath <- file.path(getwd(), base_dir, "figures")
  make_html(optim_fit, path = tmppath, template = "nea_ghl")
  rm(tmppath)
}
