library(knitr); library(kableExtra); library(flextable)

output_type = function() {
  out <- knitr::opts_knit$get("rmarkdown.pandoc.to")
  if(length(out) == 0) "html" else out
}

make_table <- function(tmp, caption = "", digits = 0, font_size = 10) {
  
  if(output_type() == "html") {
    knitr::kable(
      tmp,
      caption = caption,
      format.args = list(big.mark = ""),
      digits = digits,
      row.names = FALSE) %>%
      kableExtra::kable_styling(bootstrap_options = c("striped", "condensed"))
    
  } else if (output_type() == "docx") {
    
    FitFlextableToPage <- function(ft, pgwidth = 6){
      ft_out <- ft %>% flextable::set_table_properties(layout = "autofit") %>% flextable::autofit()
      ft_out <- flextable::width(ft_out, width = dim(ft_out)$widths*pgwidth/(flextable::flextable_dim(ft_out)$widths))
      return(ft_out)
    }
    
    ft <- flextable::flextable(tmp) %>%
      flextable::fontsize(size = font_size, part="all") %>%
      flextable::set_caption(caption = caption) %>%
      flextable::set_table_properties(layout = "autofit") %>%
      flextable::align_nottext_col(align = "center") %>%
      flextable::align_text_col(align = "left") %>%
      flextable::colformat_int(big.mark = "", na_str = "") %>%
      flextable::colformat_double(big.mark = "", digits = digits, na_str = "")
    
    FitFlextableToPage(ft)
    
  } else {
    tmp %>% 
      knitr::kable(
        format = "latex",
        caption = caption,
        booktabs = TRUE, 
        longtable = TRUE,
        row.names = FALSE,
        format.args = list(big.mark = ""),
        digits = digits) %>%
      kableExtra::kable_styling(latex_options = c("striped"))
  }
}
