## ---------------------------
##
## Script name: Run long- and short-term projections, calculate reference points, TAC, make reference point and advice figures and tables.
##
## Purpose of script: Master script to calculate reference points and all numbers required for advice. Source the entire script or run from top to a desired line (Ctrl+Alt+B on Rstudio).
##
## Instructions: Update data and optimize the model first using _RUN_THE_MODEL.R file. Once happy with the model, place all model files into a folder called latest_model in the model root (or change base_dir object which defines the folder). Once done, you can source this script and it should take care of the rest.
##
## Authors:
## Mikko Vihtakari and Daniel Howell // Institute of Marine Research, Norway
## Will Butler and Bjarki Elvarsson // Marine and Freshwater Research Institute, Iceland
## Email: mikko.vihtakari@hi.no
##
## ---------------------------

rm(list = ls())

source("0 run first.R")

# Model settings ####
base_dir <- "latest_model" # Used to load model data generated with the _RUN_THE_MODEL.R and to save long-term projection data.
ncores <- 30 # Number of cores to use for parallel operations. You could also do parallel::detectCores(logical = TRUE)). These functions do not seem memory limited.
num_project_years <- 100 # How many years to project forward
rec_start_year <- 1990 # Recruitment will be re-sampled from this year onward. Needs to be set manually due to uncertainty in recruitment estimates early in the model
rec_end_year <- 2017 # Recruitment will be re-sampled from this year backward. Needs to be set manually to exclude the 2018 recruitment spike.
harvest_rates <- c(seq(0.00, 0.1, by = 0.01),seq(0.11, 0.3, by = 0.005), seq(0.31, 1, by = 0.01)) # A sequence of harvest rates to test, typically try #seq(0.00, 1, by = 0.01).
hr_trials <- 100 # How many trials per harvest rate (each trial will have a unique recruitment series and a unique annual harvest rate sequence (if assessment error is present)).
n_catch_years <- 3 # Number of previous years to estimate average fleet catch ratios
recstep <- 1 # Time step when recruitment occurs. Irrelevant for models with one time step for year such as this.

# Long-term projections (takes 5-10 hours) ####

source("12 long-term projections.R", echo = TRUE)

## Long-term projection figures

source("13 long-term projection figures.R", echo = TRUE)

# Short-term projections (takes some minutes) ####

source("14 short-term projections.R", echo = TRUE)
